import { querySetup, queryRenderResults } from './nopaque.listenerCallbacks.js'

/**
 * Recieves a corpus analysis session via socket.io.
 * Closes the loading modal that has been opend with requestSession at the
 * start of the request.
 */
function recieveSession(client) {
  client.socket.on('corpus_analysis_init', (response) => {
    /**
     * Check if request for session was ok.
     * If OK execute callbacks and hide/show displays.
     */
    console.group('recieve session')
    if (response.code === 200) {
      console.info('corpus_analysis_init: Client recieving session/or error',
                   'codes via socket.on');
      console.info(`corpus_analysis_init: ${response.code} - ${response.msg}`);
      console.info('corpus_analysis_init: Initialization succeeded');
      console.info(response);
      // Handling hide/show of displays
      if (client.displays.init != undefined) {
        client.displays.init.element.M_Modal.close();
        client.displays.init.setVisibilityByStatus('success');
      }
    } else {
      let errorText = `Error ${response.code} - ${response.msg}`;
      if (client.displays.init.errorContainer != undefined)  {
        client.displays.init.errorContainer.innerHTML = `<p class="red-text">`
          + `<i class="material-icons tiny">error</i>${errorText}</p>`;
      }
      if (client.displays.init != undefined)  {
        client.displays.init.setVisibilityByStatus('error');
      }
      console.error(`corpus_analysis_init: ${errorText}`);
    }
    console.groupEnd();
  });
}

/**
 * Recieves the query process status before any actual results are being
 * transmitted. So it recieves error codes if a query failed or
 * was invalid etc.
 * Also prepares the result.jsList for the incoming data.
 */
function recieveQueryStatus(client) {
  client.socket.on('corpus_analysis_query', (response) => {
    /**
     * Check if issued query was ok.
     * If OK execute callbacks and hide/show displays.
     */
    console.group('corpus_analysis_query: Client recieving query process',
                  'status via socket.on');
    if (response.code === 200) {
      console.info(`corpus_analysis_query: ${response.code} - ${response.msg}`);
      console.info(response);
      // Handling hide/show of displays
      if (client.displays.query != undefined)  {
        client.displays.query.setVisibilityByStatus("success");
      }
      // executing the callbacks
      querySetup(response.payload, client);
    } else {
      let errorText = `Error ${response.payload.code} - ${response.payload.msg}`;
      if (response.payload.code == 1281) {
        errorText += ' - Invalid Query';
      }
      nopaque.flash(errorText, "error");
      if (client.displays.query.errorContainer != undefined)  {
        client.displays.query.errorContainer.innerHTML = `<p class="red-text">`+
                  `<i class="material-icons tiny">error</i> ${errorText}</p>`;
      }
      if (client.displays.query != undefined)  {
        client.displays.query.setVisibilityByStatus("error");
      }
      console.error(`corpus_analysis_query: ${errorText}`);
    }
    console.groupEnd();
  });
}

/**
 * Recieves the query data from the request and handles it.
 */
function recieveQueryData(client) {
  console.group('corpus_analysis_query_results: Client recieving or loading',
                'query data.');
  if (client.dynamicMode) {
    console.info('Client recieving query data via socket.on');
    client.socket.on('corpus_analysis_query_results', (response) => {
    console.info('Recieved chunk', response);
    queryRenderResults(response.payload, client);
    console.info('Added chunk data to results.data and rendered it with',
                 'results.jsList');
    });
  } else {
    console.info('Client loading imported query data from database.');
    queryRenderResults(client.results.data, client);
  }
  console.groupEnd();
}

// export listeners from this module
export { recieveSession, recieveQueryStatus, recieveQueryData };